{

This file is a Delphi unit (.pas & .dcu) which I converted from the C++ .h files
for the ODBC API.  It allows you to call core and level1 functions directly,
rather than going through the BDE.  It also provides Delphi types and constants
that correspond to the C types and constants found in the ODBC API.

It has undergone very limited testing; if you use it and discover bugs, please
send me email (70400,2517) and let me know what to change, and I'll upload a new
version.

You may use and redistribute this file as you please; you may even resell it.
However, Met One assumes no responsibility for software developed that uses this
unit.

}
unit ODBCApi;

Interface

const

  SQL_INVALID_HANDLE           =   -2;
  SQL_ERROR                    =   -1;
  SQL_SUCCESS                  =   0;
  SQL_SUCCESS_WITH_INFO        =   1;
  SQL_NO_DATA_FOUND            =   100;

{ Standard SQL datatypes, using ANSI type numbering }
  SQL_CHAR                     =   1;
  SQL_NUMERIC                  =   2;
  SQL_DECIMAL                  =   3;
  SQL_INTEGER                  =   4;
  SQL_SMALLINT                 =   5;
  SQL_FLOAT                    =   6;
  SQL_REAL                     =   7;
  SQL_DOUBLE                   =   8;
  SQL_VARCHAR                  =   12;

  SQL_DATE                              =  9;
  SQL_TIME                              =  10;
  SQL_TIMESTAMP                         =  11;
  SQL_LONGVARCHAR                       =  -1;
  SQL_BINARY                            =  -2;
  SQL_VARBINARY                         =  -3;
  SQL_LONGVARBINARY                     =  -4;
  SQL_BIGINT                            =  -5;
  SQL_TINYINT                           =  -6;
  SQL_BIT                               =  -7;

  SQL_INTERVAL_YEAR                      = -80;
  SQL_INTERVAL_MONTH                     = -81;
  SQL_INTERVAL_YEAR_TO_MONTH             = -82;
  SQL_INTERVAL_DAY                       = -83;
  SQL_INTERVAL_HOUR                      = -84;
  SQL_INTERVAL_MINUTE                    = -85;
  SQL_INTERVAL_SECOND                    = -86;
  SQL_INTERVAL_DAY_TO_HOUR               = -87;
  SQL_INTERVAL_DAY_TO_MINUTE             = -88;
  SQL_INTERVAL_DAY_TO_SECOND             = -89;
  SQL_INTERVAL_HOUR_TO_MINUTE            = -90;
  SQL_INTERVAL_HOUR_TO_SECOND            = -91;
  SQL_INTERVAL_MINUTE_TO_SECOND          = -92;
  SQL_UNICODE                            = -95;
  SQL_UNICODE_VARCHAR                    = -96;
  SQL_UNICODE_LONGVARCHAR                = -97;
  SQL_UNICODE_CHAR                       = SQL_UNICODE;
  SQL_TYPE_DRIVER_START                  = SQL_INTERVAL_YEAR;
  SQL_TYPE_DRIVER_END                    = SQL_UNICODE_LONGVARCHAR;

  SQL_TYPE_NULL                =   0;
  SQL_TYPE_MIN     	       =   SQL_BIT;
  SQL_TYPE_MAX                 =   SQL_VARCHAR;
  SQL_ALL_TYPES    	       =   0;

{ C datatype to SQL datatype mapping      SQL type}
  SQL_C_CHAR    = SQL_CHAR;             { CHAR, VARCHAR, DECIMAL, NUMERIC }
  SQL_C_LONG    = SQL_INTEGER;          { INTEGER                      }
  SQL_C_SHORT   = SQL_SMALLINT;         { SMALLINT                     }
  SQL_C_FLOAT   = SQL_REAL;             { REAL                         }
  SQL_C_DOUBLE  = SQL_DOUBLE;           { FLOAT, DOUBLE                }
  SQL_C_DEFAULT = 99;
  SQL_SIGNED_OFFSET      = -20;
  SQL_UNSIGNED_OFFSET    = -22;

{ C datatype to SQL datatype mapping }
  SQL_C_DATE      = SQL_DATE;
  SQL_C_TIME      = SQL_TIME;
  SQL_C_TIMESTAMP = SQL_TIMESTAMP;
  SQL_C_BINARY    = SQL_BINARY;
  SQL_C_BIT       = SQL_BIT;
  SQL_C_TINYINT   = SQL_TINYINT;
  SQL_C_SLONG     = SQL_C_LONG+SQL_SIGNED_OFFSET;    { SIGNED INTEGER   }
  SQL_C_SSHORT    = SQL_C_SHORT+SQL_SIGNED_OFFSET;   { SIGNED SMALLINT  }
  SQL_C_STINYINT  = SQL_TINYINT+SQL_SIGNED_OFFSET;   { SIGNED TINYINT   }
  SQL_C_ULONG     = SQL_C_LONG+SQL_UNSIGNED_OFFSET;  { UNSIGNED INTEGER }
  SQL_C_USHORT    = SQL_C_SHORT+SQL_UNSIGNED_OFFSET; { UNSIGNED SMALLINT}
  SQL_C_UTINYINT  = SQL_TINYINT+SQL_UNSIGNED_OFFSET; { UNSIGNED TINYINT }
  SQL_C_BOOKMARK  = SQL_C_ULONG;                     { BOOKMARK         }

{ NULL status constants.  These are used in SQLColumns, SQLColAttributes,
SQLDescribeCol, SQLDescribeParam, and SQLSpecialColumns to describe the
nullablity of a column in a table. }
  SQL_NO_NULLS                  =  0;
  SQL_NULLABLE                  =  1;
  SQL_NULLABLE_UNKNOWN          =  2;

{ Special length values }
  SQL_NULL_DATA                  = -1;
  SQL_DATA_AT_EXEC               = -2;
  SQL_NTS                        = -3;

{ SQLFreeStmt defines }
  SQL_CLOSE                      = 0;
  SQL_DROP                       = 1;
  SQL_UNBIND                     = 2;
  SQL_RESET_PARAMS               = 3;

{ SQLTransact defines }
  SQL_COMMIT                     = 0;
  SQL_ROLLBACK                   = 1;

{ SQLColAttributes defines }
  SQL_COLUMN_COUNT               = 0;
  SQL_COLUMN_NAME                = 1;
  SQL_COLUMN_TYPE                = 2;
  SQL_COLUMN_LENGTH              = 3;
  SQL_COLUMN_PRECISION           = 4;
  SQL_COLUMN_SCALE               = 5;
  SQL_COLUMN_DISPLAY_SIZE        = 6;
  SQL_COLUMN_NULLABLE            = 7;
  SQL_COLUMN_UNSIGNED            = 8;
  SQL_COLUMN_MONEY               = 9;
  SQL_COLUMN_UPDATABLE           = 10;
  SQL_COLUMN_AUTO_INCREMENT      = 11;
  SQL_COLUMN_CASE_SENSITIVE      = 12;
  SQL_COLUMN_SEARCHABLE          = 13;
  SQL_COLUMN_TYPE_NAME           = 14;
  SQL_COLUMN_TABLE_NAME          = 15;
  SQL_COLUMN_OWNER_NAME          = 16;
  SQL_COLUMN_QUALIFIER_NAME      = 17;
  SQL_COLUMN_LABEL               = 18;
  SQL_COLATT_OPT_MAX             = SQL_COLUMN_LABEL;
  SQL_COLUMN_DRIVER_START        = 1000;
  SQL_COLATT_OPT_MIN             = SQL_COLUMN_COUNT;

{ SQLColAttributes subdefines for SQL_COLUMN_UPDATABLE }
  SQL_ATTR_READONLY               = 0;
  SQL_ATTR_WRITE                  = 1;
  SQL_ATTR_READWRITE_UNKNOWN      = 2;

{ SQLColAttributes subdefines for SQL_COLUMN_SEARCHABLE }
{ These are also used by SQLGetInfo                     }
  SQL_UNSEARCHABLE               = 0;
  SQL_LIKE_ONLY                  = 1;
  SQL_ALL_EXCEPT_LIKE            = 2;
  SQL_SEARCHABLE                 = 3;

{ SQLError defines }
  SQL_NULL_HENV                  = nil;
  SQL_NULL_HDBC                  = nil;
  SQL_NULL_HSTMT                 = nil;

{ options for SQLGetStmtOption/SQLSetStmtOption }
  SQL_QUERY_TIMEOUT             =  0;
  SQL_MAX_ROWS                  =  1;
  SQL_NOSCAN                    =  2;
  SQL_MAX_LENGTH                =  3;
  SQL_ASYNC_ENABLE              =  4;
  SQL_BIND_TYPE                 =  5;
  SQL_CURSOR_TYPE               =  6;
  SQL_CONCURRENCY               =  7;
  SQL_KEYSET_SIZE               =  8;
  SQL_ROWSET_SIZE               =  9;
  SQL_SIMULATE_CURSOR           =  10;
  SQL_RETRIEVE_DATA             =  11;
  SQL_USE_BOOKMARKS             =  12;
  SQL_GET_BOOKMARK              =  13;      {      GetStmtOption Only }
  SQL_ROW_NUMBER                =  14;      {      GetStmtOption Only }
  SQL_STMT_OPT_MAX              =  SQL_ROW_NUMBER;
  SQL_STMT_OPT_MIN              =  SQL_QUERY_TIMEOUT;


{ SQL_QUERY_TIMEOUT options }
  SQL_QUERY_TIMEOUT_DEFAULT     =  0;

{ SQL_MAX_ROWS options }
  SQL_MAX_ROWS_DEFAULT          =  0;

{ SQL_NOSCAN options }
  SQL_NOSCAN_OFF                =  0;     {      1.0 FALSE }
  SQL_NOSCAN_ON                 =  1;     {      1.0 TRUE }
  SQL_NOSCAN_DEFAULT            =  SQL_NOSCAN_OFF;

{ SQL_MAX_LENGTH options }
  SQL_MAX_LENGTH_DEFAULT        =  0;

{ SQL_ASYNC_ENABLE options }
  SQL_ASYNC_ENABLE_OFF          =  0;
  SQL_ASYNC_ENABLE_ON           =  1;
  SQL_ASYNC_ENABLE_DEFAULT      =  SQL_ASYNC_ENABLE_OFF;

{ SQL_BIND_TYPE options }
  SQL_BIND_BY_COLUMN            =  0;
  SQL_BIND_TYPE_DEFAULT         =  SQL_BIND_BY_COLUMN;  { Default value }

{ SQL_CONCURRENCY options }
  SQL_CONCUR_READ_ONLY          =  1;
  SQL_CONCUR_LOCK               =  2;
  SQL_CONCUR_ROWVER             =  3;
  SQL_CONCUR_VALUES             =  4;
  SQL_CONCUR_DEFAULT            =  SQL_CONCUR_READ_ONLY; { Default value }

{ SQL_CURSOR_TYPE options }
  SQL_CURSOR_FORWARD_ONLY       =  0;
  SQL_CURSOR_KEYSET_DRIVEN      =  1;
  SQL_CURSOR_DYNAMIC            =  2;
  SQL_CURSOR_STATIC             =  3;
  SQL_CURSOR_TYPE_DEFAULT       =  SQL_CURSOR_FORWARD_ONLY;

{ SQL_ROWSET_SIZE options }
  SQL_ROWSET_SIZE_DEFAULT       =  1;

{ SQL_KEYSET_SIZE options }
  SQL_KEYSET_SIZE_DEFAULT       =  0;

{ SQL_SIMULATE_CURSOR options }
  SQL_SC_NON_UNIQUE             =  0;
  SQL_SC_TRY_UNIQUE             =  1;
  SQL_SC_UNIQUE                 =  2;

{ SQL_RETRIEVE_DATA options }
  SQL_RD_OFF                    =  0;
  SQL_RD_ON                     =  1;
  SQL_RD_DEFAULT                =  SQL_RD_ON;

{ SQL_USE_BOOKMARKS options }
  SQL_UB_OFF                    =  0;
  SQL_UB_ON                     =  1;
  SQL_UB_DEFAULT                =  SQL_UB_OFF;

{ options for SQLSetConnectOption/SQLGetConnectOption }
  SQL_ACCESS_MODE               =  101;
  SQL_AUTOCOMMIT                =  102;
  SQL_LOGIN_TIMEOUT             =  103;
  SQL_OPT_TRACE                 =  104;
  SQL_OPT_TRACEFILE             =  105;
  SQL_TRANSLATE_DLL             =  106;
  SQL_TRANSLATE_OPTION          =  107;
  SQL_TXN_ISOLATION             =  108;
  SQL_CURRENT_QUALIFIER         =  109;
  SQL_ODBC_CURSORS              =  110;
  SQL_QUIET_MODE                =  111;
  SQL_PACKET_SIZE               =  112;
  SQL_CONN_OPT_MAX              =  SQL_PACKET_SIZE;
  SQL_CONNECT_OPT_DRVR_START    =  1000;
  SQL_CONN_OPT_MIN              =  SQL_ACCESS_MODE;

{ SQL_ACCESS_MODE options }
  SQL_MODE_READ_WRITE           =  0;
  SQL_MODE_READ_ONLY            =  1;
  SQL_MODE_DEFAULT              =  SQL_MODE_READ_WRITE;

{ SQL_AUTOCOMMIT options }
  SQL_AUTOCOMMIT_OFF            =  0;
  SQL_AUTOCOMMIT_ON             =  1;
  SQL_AUTOCOMMIT_DEFAULT        =  SQL_AUTOCOMMIT_ON;

{ SQL_LOGIN_TIMEOUT options }
  SQL_LOGIN_TIMEOUT_DEFAULT     =  15;

{ SQL_OPT_TRACE options }
  SQL_OPT_TRACE_OFF             =  0;
  SQL_OPT_TRACE_ON              =  1;
  SQL_OPT_TRACE_DEFAULT         =  SQL_OPT_TRACE_OFF;
  SQL_OPT_TRACE_FILE_DEFAULT    =  '\SQL.LOG';

{ SQL_ODBC_CURSORS options }
  SQL_CUR_USE_IF_NEEDED         =  0;
  SQL_CUR_USE_ODBC              =  1;
  SQL_CUR_USE_DRIVER            =  2;
  SQL_CUR_DEFAULT               =  SQL_CUR_USE_DRIVER;

{ SQLBindParam Extension Options }
  SQL_DEFAULT_PARAM            = -5;
  SQL_IGNORE                   = -6;
  SQL_LEN_DATA_AT_EXEC_OFFSET  = -100;
  SQL_PARAM_TYPE_UNKNOWN       = 0;
  SQL_PARAM_INPUT              = 1;
  SQL_PARAM_INPUT_OUTPUT       = 2;
  SQL_RESULT_COL               = 3;
  SQL_PARAM_OUTPUT             = 4;
  SQL_RETURN_VALUE             = 5;

{ Defines used by Driver Manager when mapping SQLSetParam to SQLBindParameter}
  SQL_PARAM_TYPE_DEFAULT       = SQL_PARAM_INPUT_OUTPUT;
  SQL_SETPARAM_VALUE_MAX       = -1;

{ Defines for SQLSetPos }
  SQL_ENTIRE_ROWSET            = 0;

{ Operations in SQLSetPos }
  SQL_POSITION                 = 0;               {      1.0 FALSE }
  SQL_REFRESH                  = 1;               {      1.0 TRUE }
  SQL_UPDATE                   = 2;
  SQL_DELETE                   = 3;
  SQL_ADD                      = 4;

{ Lock options in SQLSetPos }
  SQL_LOCK_NO_CHANGE           = 0;               {      1.0 FALSE }
  SQL_LOCK_EXCLUSIVE           = 1;               {      1.0 TRUE }
  SQL_LOCK_UNLOCK              = 2;

{SQLExtendedFetch Options}
  SQL_FETCH_NEXT                   = 1;
  SQL_FETCH_FIRST                  = 2;
  SQL_FETCH_LAST                   = 3;
  SQL_FETCH_PRIOR                  = 4;
  SQL_FETCH_ABSOLUTE               = 5;
  SQL_FETCH_RELATIVE               = 6;
  SQL_FETCH_BOOKMARK               = 8;

{ SQLExtendedFetch "rgfRowStatus" element values }
  SQL_ROW_SUCCESS                  = 0;
  SQL_ROW_DELETED                  = 1;
  SQL_ROW_UPDATED                  = 2;
  SQL_ROW_NOROW                    = 3;
  SQL_ROW_ADDED                    = 4;
  SQL_ROW_ERROR                    = 5;

type

{ SQL portable types for C }
  UCHAR    = Byte;
  SCHAR    = Byte;
  SDWORD   = Integer;
  SWORD    = Smallint;
  UDWORD   = Cardinal;
  UWORD    = Word;

  SLONG    = Integer;
  SSHORT   = Smallint;
  ULONG    = Cardinal;
  USHORT   = Word;
  SDOUBLE  = Double;
  LDOUBLE  = Double; { long double == short double in Win32 }
  SFLOAT   = Single;

  PTR      = Pointer;

  HENV     = Pointer;
  HDBC     = Pointer;
  HSTMT    = Pointer;

  RETCODE  = SSHORT;

  SQLCHAR  = UCHAR;
  SQLSCHAR = SCHAR;
  SQLINTEGER = SDWORD;
  SQLSMALLINT = SWORD;
  SQLUINTEGER = UDWORD;
  SQLUSMALLINT = UWORD;

  SQLPOINTER = Pointer;

  SQLHENV      = HENV;
  SQLHDBC      = HDBC;
  SQLHSTMT     = HSTMT;

  SQLRETURN    = SQLSMALLINT;

{ transfer types for DATE, TIME, TIMESTAMP }
  DATE_STRUCT = record 
        year: SQLSMALLINT;
        month: SQLUSMALLINT;
        day: SQLUSMALLINT;
  end;

  TIME_STRUCT = record
        hour: SQLUSMALLINT;
        minute: SQLUSMALLINT;
        second: SQLUSMALLINT;
  end;

  TIMESTAMP_STRUCT = record
        year: SQLSMALLINT;
        month: SQLUSMALLINT;
        day: SQLUSMALLINT;
        hour: SQLUSMALLINT;
        minute: SQLUSMALLINT;
        second: SQLUSMALLINT;
        fraction: SQLUINTEGER;
  end;

{Core ODBC functions}

function SQLAllocConnect(
    henv: SQLHENV;
    var phdbc: SQLHDBC): SQLRETURN; stdcall; external 'ODBC32.DLL' index 1;

function SQLAllocEnv(var phenv: SQLHENV): SQLRETURN; stdcall;
    external 'ODBC32.DLL' index 2;

function SQLAllocStmt(
    hdbc: SQLHDBC;
    var phstmt: SQLHSTMT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 3;

function SQLBindCol(
    hstmt: SQLHSTMT;
    icol: SQLUSMALLINT;
    fCType: SQLSMALLINT;
    rgbValue: SQLPOINTER;
    cbValueMax: SQLINTEGER;
    var pcbValue: SQLINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 4;

function SQLCancel(hstmt: SQLHSTMT): SQLRETURN; stdcall;
    external 'ODBC32.DLL' index 5;

function SQLColAttributes(
    hstmt: SQLHSTMT;
    icol: SQLUSMALLINT;
    fDescType: SQLUSMALLINT;
    rgbDesc: SQLPOINTER;
    cbDescMax: SQLSMALLINT;
    var pcbDesc: SQLSMALLINT;
    var pfDesc: SQLINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 6;

function SQLConnect(
    hdbc: SQLHDBC;
    szDSN: PChar;
    cbDSN: SQLSMALLINT;
    szUID: PChar;
    cbUID: SQLSMALLINT;
    szAuthStr: PChar;
    cbAuthStr: SQLSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 7;

function SQLDescribeCol(
    hstmt: SQLHSTMT;
    icol: SQLUSMALLINT;
    szColName: PChar;
    cbColNameMax: SQLSMALLINT;
    var pcbColName: SQLSMALLINT;
    var pfSqlType: SQLSMALLINT;
    var pcbColDef: SQLUINTEGER;
    var pibScale: SQLSMALLINT;
    var pfNullable: SQLSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 8;

function SQLDisconnect(hdbc: SQLHDBC): SQLRETURN; stdcall;
    external 'ODBC32.DLL' index 9;

function SQLError(
    henv: SQLHENV;
    hdbc: SQLHDBC;
    hstmt: SQLHSTMT;
    szSqlState: PChar;
    var pfNativeError: SQLINTEGER;
    szErrorMsg: PChar;
    cbErrorMsgMax: SQLSMALLINT;
    var pcbErrorMsg: SQLSMALLINT): SQLRETURN; stdcall;
    external 'ODBC32.DLL' index 10;

function SQLExecDirect(
    hstmt: SQLHSTMT;
    szSqlStr: PChar;
    cbSqlStr: SQLINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 11;

function SQLExecute(hstmt: SQLHSTMT): SQLRETURN; stdcall;
    external 'ODBC32.DLL' index 12;

function SQLFetch(hstmt: SQLHSTMT): SQLRETURN; stdcall;
    external 'ODBC32.DLL' index 13;

function SQLFreeConnect(hdbc: SQLHDBC): SQLRETURN; stdcall;
    external 'ODBC32.DLL' index 14;

function SQLFreeEnv(henv: SQLHENV): SQLRETURN; stdcall;
    external 'ODBC32.DLL' index 15;

function SQLFreeStmt(
    hstmt: SQLHSTMT;
    fOption: SQLUSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 16;

function SQLGetCursorName(
    hstmt: SQLHSTMT;
    szCursor: PChar;
    cbCursorMax: SQLSMALLINT;
    var pcbCursor: SQLSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 17;

function SQLNumResultCols(
    hstmt: SQLHSTMT;
    var pccol: SQLSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 18;

function SQLPrepare(
    hstmt: SQLHSTMT;
    szSqlStr: PChar;
    cbSqlStr: SQLINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 19;

function SQLRowCount(
    hstmt: SQLHSTMT;
    var pcrow: SQLINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 20;

function SQLSetCursorName(
    hstmt: SQLHSTMT;
    szCursor: PChar;
    cbCursor: SQLSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 21;

function SQLTransact(
    henv: SQLHENV;
    hdbc: SQLHDBC;
    fType: SQLUSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 23;

{Level 1 functions}

function SQLColumns(
    hstmt: SQLHSTMT;
    szCatalogName: PChar;
    cbCatalogName: SQLSMALLINT;
    szSchemaName: PChar;
    cbSchemaName: SQLSMALLINT;
    szTableName: PChar;
    cbTableName: SQLSMALLINT;
    szColumnName: PChar;
    cbColumnName: SQLSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 40;

function SQLGetConnectOption(
    hdbc: SQLHDBC;
    fOption: SQLUSMALLINT;
    pvParam: SQLPOINTER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 42;

function SQLGetData(
    hstmt: SQLHSTMT;
    icol: SQLUSMALLINT;
    fCType: SQLSMALLINT;
    rgbValue: SQLPOINTER;
    cbValueMax: SQLINTEGER;
    var pcbValue: SQLINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 43;

function SQLGetFunctions(
    hdbc: SQLHDBC;
    fFunction: SQLUSMALLINT;
    var pfExists: SQLUSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 44;

function SQLGetInfo(
    hdbc: SQLHDBC;
    fInfoType: SQLUSMALLINT;
    rgbInfoValue: SQLPOINTER;
    cbInfoValueMax: SQLSMALLINT;
    var pcbInfoValue: SQLSMALLINT): SQLRETURN; stdcall;
    external 'ODBC32.DLL' index 45;

function SQLGetStmtOption(
    hstmt: SQLHSTMT;
    fOption: SQLUSMALLINT;
    pvParam: SQLPOINTER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 46;

function SQLGetTypeInfo(
    hstmt: SQLHSTMT;
    fSqlType: SQLSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 47;

function SQLParamData(
    hstmt: SQLHSTMT;
    var prgbValue: SQLPOINTER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 48;

function SQLPutData(
    hstmt: SQLHSTMT;
    rgbValue: SQLPOINTER;
    cbValue: SQLINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 49;

function SQLSetConnectOption(
    hdbc: SQLHDBC;
    fOption: SQLUSMALLINT;
    vParam: SQLUINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 50;

function SQLSetPos(
    hstmt:   SQLHSTMT;
    irow:    SQLUSMALLINT;
    fOption: SQLUSMALLINT;
    fLock:   SQLUSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 68;

function SQL_POSITION_TO(
    hstmt: SQLHSTMT;
    irow: SQLUSMALLINT): SQLRETURN;
function SQL_LOCK_RECORD(
    hstmt: SQLHSTMT;
    irow, fLock: SQLUSMALLINT): SQLRETURN;
function SQL_REFRESH_RECORD(
    hstmt: SQLHSTMT;
    irow, fLock: SQLUSMALLINT): SQLRETURN;
function SQL_UPDATE_RECORD(
    hstmt: SQLHSTMT;
    irow: SQLUSMALLINT): SQLRETURN;
function SQL_DELETE_RECORD(
    hstmt: SQLHSTMT;
    irow: SQLUSMALLINT): SQLRETURN;
function SQL_ADD_RECORD(
    hstmt: SQLHSTMT;
    irow: SQLUSMALLINT): SQLRETURN;


function SQLSetStmtOption(
    hstmt: SQLHSTMT;
    fOption: SQLUSMALLINT;
    vParam: SQLUINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 51;

function SQLSpecialColumns(
    hstmt: SQLHSTMT;
    fColType: SQLUSMALLINT;
    szCatalogName: PChar;
    cbCatalogName: SQLSMALLINT;
    szSchemaName: PChar;
    cbSchemaName: SQLSMALLINT;
    szTableName: PChar;
    cbTableName: SQLSMALLINT;
    fScope: SQLUSMALLINT;
    fNullable: SQLUSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 52;

function SQLStatistics(
    hstmt: SQLHSTMT;
    szCatalogName: PChar;
    cbCatalogName: SQLSMALLINT;
    szSchemaName: PChar;
    cbSchemaName: SQLSMALLINT;
    szTableName: PChar;
    cbTableName: SQLSMALLINT;
    fUnique: SQLUSMALLINT;
    fAccuracy: SQLUSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 53;

function SQLTables(
    hstmt: SQLHSTMT;
    szCatalogName: PChar;
    cbCatalogName: SQLSMALLINT;
    szSchemaName: PChar;
    cbSchemaName: SQLSMALLINT;
    szTableName: PChar;
    cbTableName: SQLSMALLINT;
    szTableType: PChar;
    cbTableType: SQLSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 54;

 {SOME of the Level 2 functions}

 function SQL_LEN_DATA_AT_EXEC(length: SQLINTEGER): SQLINTEGER;

 function SQLBindParameter(
    hstmt: SQLHSTMT;
    ipar: SQLUSMALLINT;
    fParamType: SQLSMALLINT;
    fCType: SQLSMALLINT;
    fSqlType: SQLSMALLINT;
    cbColDef: SQLUINTEGER;
    ibScale: SQLSMALLINT;
    rgbValue: SQLPOINTER;
    cbValueMax: SQLINTEGER;
    var pcbValue: SQLINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 72;

  function SQLDescribeParam(
    hstmt: SQLHSTMT;
    ipar: SQLUSMALLINT;
    var pfSqlType: SQLSMALLINT;
    var pcbParamDef: SQLUINTEGER;
    var pibScale: SQLSMALLINT;
    var pfNullable: SQLSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 58;

 function SQLExtendedFetch(
    hstmt: SQLHSTMT;
    fFetchType: SQLUSMALLINT;
    irow: SQLINTEGER;
    var pcrow: SQLUINTEGER;
    var rgfRowStatus: SQLUSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 59;

function SQLNativeSql(
    hdbc: SQLHDBC;
    szSqlStrIn: PChar;
    cbSqlStrIn: SQLINTEGER;
    szSqlStr: PChar;
    cbSqlStrMax: SQLINTEGER;
    var pcbSqlStr: SQLINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 62;

function SQLNumParams(
    hstmt: SQLHSTMT;
    var pcpar: SQLSMALLINT): SQLRETURN; stdcall; external 'ODBC32.DLL' index 63;

function SQLParamOptions(
    hstmt: SQLHSTMT;
    crow: SQLUINTEGER;
    var pirow: SQLUINTEGER): SQLRETURN; stdcall; external 'ODBC32.DLL' index 64;

Implementation

function SQL_LEN_DATA_AT_EXEC(length: SQLINTEGER): SQLINTEGER;
begin
  SQL_LEN_DATA_AT_EXEC:= -length+SQL_LEN_DATA_AT_EXEC_OFFSET;
end;

function SQL_POSITION_TO(hstmt: SQLHSTMT; irow: SQLUSMALLINT): SQLRETURN;
begin
  SQL_POSITION_TO := SQLSetPos(hstmt,irow,SQL_POSITION,SQL_LOCK_NO_CHANGE);
end;

function SQL_LOCK_RECORD(hstmt: SQLHSTMT; irow,fLock: SQLUSMALLINT): SQLRETURN;
begin
  SQL_LOCK_RECORD := SQLSetPos(hstmt,irow,SQL_POSITION,fLock);
end;

function SQL_REFRESH_RECORD(hstmt: SQLHSTMT;
                            irow,fLock: SQLUSMALLINT): SQLRETURN;
begin
  SQL_REFRESH_RECORD := SQLSetPos(hstmt,irow,SQL_REFRESH,fLock);
end;

function SQL_UPDATE_RECORD(hstmt: SQLHSTMT; irow: SQLUSMALLINT): SQLRETURN;
begin
  SQL_UPDATE_RECORD := SQLSetPos(hstmt,irow,SQL_UPDATE,SQL_LOCK_NO_CHANGE);
end;

function SQL_DELETE_RECORD(hstmt: SQLHSTMT; irow: SQLUSMALLINT): SQLRETURN;
begin
  SQL_DELETE_RECORD := SQLSetPos(hstmt,irow,SQL_DELETE,SQL_LOCK_NO_CHANGE);
end;

function SQL_ADD_RECORD(hstmt: SQLHSTMT; irow: SQLUSMALLINT): SQLRETURN;
begin
  SQL_ADD_RECORD := SQLSetPos(hstmt,irow,SQL_ADD,SQL_LOCK_NO_CHANGE);
end;

end.

